#
# MailmanTool.py
#
# Herramienta para integrar un servidor Mailman en Plone
#
# Alejandro Álvarez Ayllón - 26/08/2006
#

from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.utils import UniqueObject, SimpleItemWithProperties
from Products.CMFCore.permissions import ManagePortal
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass, DTMLFile

from config import dtmlDir

import os

# Clase
class MailmanTool(UniqueObject, SimpleItemWithProperties):
    """Herramienta para integrar un servidor Mailman en Plone"""

    __implements__ = (SimpleItemWithProperties.__implements__)

    id        = 'mailman_tool'
    meta_type = 'mailman_tool'

    manage_options= SimpleItemWithProperties.manage_options + \
                    ({'label' : 'Mail lists',
                     'action': 'manage_mailLists'},
                     )

    manage_mailLists = DTMLFile('manage_mailLists', dtmlDir)

    # Seguridad
    security  = ClassSecurityInfo()

    security.declareProtected(ManagePortal, 'manage_editProperties')
    security.declareProtected(ManagePortal, 'manage_changeProperties')
    security.declareProtected(ManagePortal, 'manage_propertiesForm')
    security.declareProtected(ManagePortal, 'manage_mailLists')

    # Propiedades
    _properties = (
        {'id': 'domain', 'type': 'string', 'mode': 'w',
         'label': 'Domain',},
        {'id': 'mailman_binaries', 'type': 'string', 'mode': 'w',
         'label': 'Mailman binaries',},
        {'id': 'mailman_archives', 'type': 'string', 'mode': 'w',
         'label': 'Mailman archives',},
        {'id': 'mailman_lang', 'type': 'string', 'mode': 'w',
         'label': 'Default list language',},
        {'id': 'mailman_admin', 'type': 'string', 'mode': 'w',
         'label': 'Mailman administrator mail',},
        {'id': 'mailman_adminpasswd', 'type': 'string', 'mode': 'w',
         'label': 'Mailman administrator password',},
        {'id': 'mailman_rmarchive', 'type': 'boolean', 'mode': 'w',
         'label': 'Remove mail list archive',}, 
        )

    # Valores por defecto de las propiedades
    domain              = 'example.com'
    mailman_binaries    = '/usr/lib/mailman/bin'
    mailman_archives    = '/var/lib/mailman/archives'
    mailman_lang        = 'en'
    mailman_admin       = 'admin@localdomain.com'
    mailman_adminpasswd = 'admin1234'
    mailman_rmarchive   = False

    # Añadir una nueva lista
    def newList(self, listName, listAdmin = None, listAdminPasswd = None):
        """
        Descripción: Crea una nueva lista.
        Parámetros:  listName        = Nombre de la lista.
                     listAdmin       = Correo del administrador.
                     listAdminPasswd = Contraseña de administración.
        Devuelve:    Nada.
        """

        # Mail del administrador y su contraseña
        if listAdmin is None:
            listAdmin = self.mailman_admin
        if listAdminPasswd is None:
            listAdminPasswd = self.mailman_adminpasswd

        # Comprueba que no existe
        if self.listExists(listName):
            raise IOError('The mail list already exists')

        # Ejecuta el comando
        bin_path = '%s/newlist' % self.mailman_binaries
        fail = os.spawnlp(os.P_WAIT, bin_path, 'newlist',
                   '-q', # No preguntar
                   '-l',
                   self.mailman_lang, # Idioma preferente
                   listName, # Nombre de la lista
                   listAdmin, # Administrador
                   listAdminPasswd # Contraseña del administrador
                   )

        if fail != 0:
            raise IOError('The mail list can not be created: %s (%s)' % (listName,
                                                                         listAdmin))

    # Eliminar una lista
    def removeList(self, listName, removeArchive = False):
        """
        Descripción: Elimina una lista.
        Parámetros:  listName      = El nombre de la lista a eliminar.
                     removeArchive = Indica si eliminar el histórico de la lista.
        Devuelve:    Nada.
        """

        # Determinar si eliminar el histórico
        if removeArchive is None:
            removeArchive = self.mailman_rmarchive

        # Ejecutar el comando
        bin_path = '%s/rmlist' % self.mailman_binaries
        
        if removeArchive:
            fail = os.spawnlp(os.P_WAIT, bin_path, bin_path,
                              listName)
        else:
            fail = os.spawnlp(os.P_WAIT, bin_path, bin_path,
                              '-a', # Eliminar el histórico
                              listName
                              )

        if fail:
            raise IOError('The mail list can not be removed')

    # Devuelve una lista con las listas existentes
    def listsList(self):
        """
        Descripción: Devuelve una lista con las listas de correo existentes.
        Parámetros:  Ninguno.
        Devuelve:    Una lista con todas las listas de correo existentes.
        """

        bin_path = '%s/list_lists -b' % self.mailman_binaries
        # Ejecutamos empleando tuberías para obtener la salida
        pipe = os.popen(bin_path, 'r')

        if pipe is None:
            raise IOError('Can not get lists list')

        lists = []

        for line in pipe:
            lists.append(line.strip())

        pipe.close()

        return lists

    # Indica si una lista existe
    def listExists(self, listName):
        """
        Descripción: Indica si una lista existe.
        Parámetros:  listName = Nombre de la lista a comprobar.
        Devuelve:    True si existe la lista. False si no.
        """

        return (listName in self.listsList())

    # Añadir usuarios a una lista
    def addMembers(self, listName, usersMails):
        """
        Descripción: Añade nuevos usuarios a una lista.
        Parámetros:  listName   = Nombre de la lista donde se añadirán.
                     usersMails = Lista/vector/tupla con los correos a añadir.
        Devuelve:    Nada.
        """

        bin_path = '%s/add_members' % self.mailman_binaries

        # Ejecutar empleando tuberías para pasar los usuarios
        cmd = '%s -r - %s' % (bin_path, listName)
        pipe = os.popen(cmd, 'w')

        if pipe is None:
            raise IOError('Can not add users')

        for i in usersMails:
            pipe.write('%s\n' % i)
        pipe.flush()

        pipe.close()

    # Elimina usuarios de una lista
    def removeMembers(self, listName, usersMails):
        """
        Descripción: Elimina los usuarios indicados de la lista.
        Parámetros:  listName   = Nombre de la lista de dónde se eliminarán.
                     usersMails = Lista/vector/tupla con los correos que se borrarán.
        Devuelve:    Nada.
        """

        bin_path = '%s/remove_members' % self.mailman_binaries
        # Ejecutar empleando tuberías
        cmd = '%s -f - %s' % (bin_path, listName)
        pipe = os.popen(cmd, 'w')

        if pipe is None:
            raise IOError('Can not remove users')

        for i in usersMails:
            pipe.write('%s\n' % i)
        pipe.flush()

        pipe.close()

    # Actualiza la lista de suscriptores de una lista
    def updateMembers(self, listName, usersMails):
        """
        Descripción: Actualiza la lista de suscriptores de una lista. Elimina los que no están
                     y mete los nuevos.
        Parámetros:  listName   = Nombre de la lista a actualizar.
                     usersMails = Lista/vector/tupla con todos los correos de la lista.
        Devuelve:    Nada.
        """

        bin_path = '%s/sync_members' % self.mailman_binaries
        # Ejecutar empleando tuberías
        cmd = '%s -f - %s' % (bin_path, listName)
        pipe = os.popen(cmd, 'w')

        if pipe is None:
            raise IOError('Can not update members')

        for i in usersMails:
            pipe.write('%s\n' % i)
        pipe.flush()
        
        pipe.close()

    # Devuelve la lista de suscriptores de una lista
    def membersList(self, listName):
        """
        Descripción: Devuelve la lista de suscriptores de una lista.
        Parámetros:  listName = Nombre de la lista de correo.
        Devuelve:    Una lista con todos los correos de los suscriptores.
        """

        bin_path = '%s/list_members' % self.mailman_binaries
        # Ejecutar empleando tuberías
        cmd = '%s -p %s' % (bin_path, listName)
        pipe = os.popen(cmd, 'r')

        if pipe is None:
            raise IOError('Can not get member list')

        # Leemos
        lists = []

        for i in pipe:
            lists.append(i.strip())

        pipe.close()

        return lists

    # Devuelve el archivo pedido
    def listArchive(self, listName, fileName):
        """
        Descripción: Devuelve el fichero pedido del archivo de la lista.
        Parámetros:  listName = Nombre de la lista de correo.
                     fileName = Nombre del fichero del archivo.
        Devuelve:    El código HTML del fichero del archivo de listName.
                     Elimina la cabecera y el pie del documento.
                     NO ADAPTA LOS ENLACES.
        """

        arch_path = '%s/public/%s' % (self.mailman_archives, listName.lower())

        if len(fileName) == 0:
            fileName = 'index.html'

        # Obtenemos el contenido del archivo
        arch_path = '%s/%s' % (arch_path, fileName)
        
        html = open(arch_path, 'r')
        content = html.read()
        html.close()

        # Buscamos el inicio del contenido
        lcontent = content.lower()
        
        i = lcontent.find('<body ')
        start = lcontent.find('>', i) + 1
        end = lcontent.find('</body>')

        # Devolvemos
        if start != -1 and end != -1:
            return content[start:end]
        else:
            return content
    
    #### Métodos para la administración ####
    
    security.declareProtected(ManagePortal, 'manage_updateList')
    def manage_updateList(self, REQUEST = None):
        """
        Descripción: Actualiza una lista desde el panel.
        Parámetros:  REQUEST = El objeto request de la solicitud.
        Devuelve:    La página de administración de las listas.
        """

        if REQUEST is not None:

            # Valores de los campos
            for i in REQUEST.items():
                if i[0] == 'listName':
                    listName = i[1]
                elif i[0] == 'listMembers':
                    listMembers = i[1]

            # Volver al formulario
            if REQUEST.has_key('delete'):
                self.removeList(listName)
                return self.manage_mailLists(manage_tabs_message = '%s deleted' % listName)
            else:
                self.updateMembers(listName, listMembers.splitlines())
                return self.manage_mailLists(manage_tabs_message = '%s updated' % listName)

    security.declareProtected(ManagePortal, 'manage_newList')
    def manage_newList(self, REQUEST = None):
        """
        Descripción: Crea una nueva lista desde el panel de administración.
        Parámetros:  REQUEST = El objeto request de la solicitud.
        Devuelve:    La página de administración de las listas.
        """

        if REQUEST is not None:
            # Crear la lista
            fields = REQUEST.items()

            for i in fields:
                if i[0] == 'listName':
                    listName = i[1]

            self.newList(listName)

            # Volver al formulario
            return self.manage_mailLists(manage_tabs_message = '%s created' % listName)


# Inicializar
InitializeClass(MailmanTool)
